import pygame
from pygame import Font, Surface

from ten_drops import FONT_PATH, SCREEN


class Cover:
    TitleFont = Font(FONT_PATH, size=120)
    TextFont = Font(FONT_PATH, size=42)

    def __init__(self):
        self._update_image()

    def _update_image(self):
        width = SCREEN.get_width()
        height = SCREEN.get_height()
        cover_surface = Surface((width, height), pygame.SRCALPHA)
        cover_surface.fill((255, 255, 255, 30))

        title_image = self.TitleFont.render("ten drops", True, (51, 255, 255))
        cover_surface.blit(
            title_image, ((width - title_image.get_width()) / 2, height / 10)
        )

        text_image = self.TextFont.render(
            "A ten drops game written in pygame.", True, (32, 32, 32)
        )

        cover_surface.blit(
            text_image, ((width - text_image.get_width()) / 2, height / 3)
        )

        self.image = cover_surface
        self.rect = self.image.get_rect()

    def draw(self):
        SCREEN.blit(self.image, self.rect)



import random
import pygame

from collections import namedtuple
from pygame.sprite import Group, groupcollide, GroupSingle

from ten_drops import (
    SCREEN,
    PLAYGROUND,
    BACKGROUND,
    GRID_SIZE,
    PLAYGROUND_OFFSET,
    PLAYGROUND_LENGTH,
    HP_SOUND,
    GROW_SOUND,
)
from ten_drops.button import StartButton, AboutButton, ExitButton
from ten_drops.cover import Cover
from ten_drops.drop import Drop, DummyDrop
from ten_drops.droplet import Droplet
from ten_drops.notification import Notice, NoticeType
from ten_drops.panel import Level, Score, HP

LevelDesign = namedtuple("LevelDesign", "state0, state1, state2, state3")
Levels = [
    LevelDesign(2, 5, 8, 9),
    LevelDesign(2, 6, 7, 8),
    LevelDesign(3, 7, 7, 7),
    LevelDesign(3, 7, 6, 6),
    LevelDesign(4, 7, 6, 5),
    LevelDesign(4, 8, 6, 5),
    LevelDesign(4, 8, 5, 5),
    LevelDesign(5, 9, 3, 5),
    LevelDesign(6, 9, 4, 5),
]


class Game:
    level: int
    score: int
    hp: int
    combo: int

    def __init__(self):
        self.start_game = False
        self.run = True

        self.drops: Group = Group()
        self.dummy_drops: Group = Group()
        self.droplets: Group = Group()
        self.panel: Group = Group()
        self.buttons: Group = Group()
        self.notifications: GroupSingle = GroupSingle()

        self.cover = Cover()

        self.clock = pygame.time.Clock()

    def _init_game_data(self):
        self.level = 1
        self.score = 0
        self.hp = 10
        self.combo = 1

    def _init_grid(self):
        used_positions = set()

        if self.level - 1 >= len(Levels):
            level = Levels[random.randint(-5, -1)]
        else:
            level = Levels[self.level - 1]

        for state, count in enumerate(level):
            for _ in range(count):
                while True:
                    row = random.randint(0, GRID_SIZE - 1)
                    col = random.randint(0, GRID_SIZE - 1)
                    if (row, col) not in used_positions:
                        used_positions.add((row, col))
                        Drop(row, col, state, self.drops)
                        break

    def _init_button(self):
        StartButton(self.buttons)
        AboutButton(self.buttons)
        ExitButton(self.buttons)

    def _init_panel(self):
        Level(self.level, self.panel)
        Score(self.score, self.panel)
        HP(self.hp, self.panel)

    def _init_dummy_drops(self):
        for i in range(GRID_SIZE):
            for j in range(GRID_SIZE):
                DummyDrop(i, j, self.dummy_drops)

    def notify(self, _type):
        if _type == NoticeType.success:
            text = "     Well Done!\n     + 2 drops\n\n     Next Level"
        elif _type == NoticeType.failed:
            text = f"You Lost\nYour Score: {self.score}"
        else:
            text = (
                "The game and water drop assets are \n"
                'from the Flash game "Splash Back".\n'
                "\n\n\nAuthor: chyok\n"
                "Email : chyok@hotmail.com\nGithub: https://github.com/chyok\n"
                "\nImplemented using pygame-ce."
            )

        Notice(_type, text, self.notifications)

    def start(self):
        self._init_game_data()
        self._init_button()
        self._init_panel()
        self._init_dummy_drops()

        last_hover_rect = pygame.Rect(0, 0, 0, 0)

        while self.run:
            self.clock.tick(30)

            for event in pygame.event.get():

                if event.type == pygame.QUIT:
                    self.run = False
                    break

                if event.type == pygame.MOUSEBUTTONDOWN:
                    mouse_x, mouse_y = pygame.mouse.get_pos()
                    if len(self.notifications) > 0:
                        self.notifications.empty()
                        continue

                    for i in self.drops:
                        if (
                            i.rect.collidepoint(mouse_x, mouse_y)
                            and len(self.droplets) == 0
                        ):
                            self.combo = 1
                            i.click()
                            self.hp = self.hp - 1
                            if i.need_diffuse:
                                self.score += 10
                                Droplet.diffusion(i.row, i.col, self.droplets)

                    if len(self.droplets) == 0 and self.start_game:
                        for j in self.dummy_drops:
                            if j.rect.collidepoint(mouse_x, mouse_y):
                                if (j.row, j.col) not in (
                                    (i.row, i.col) for i in self.drops
                                ):
                                    GROW_SOUND.play()
                                    Drop(j.row, j.col, 0, self.drops)
                                    self.hp -= 1
                                break

                    for i in self.buttons:
                        if i.rect.collidepoint(mouse_x, mouse_y):
                            if isinstance(i, StartButton):
                                self.drops.empty()
                                self.droplets.empty()
                                self._init_game_data()
                                self._init_grid()
                                self.start_game = True
                            elif isinstance(i, AboutButton):
                                self.notify(NoticeType.about)
                            elif isinstance(i, ExitButton):
                                self.run = False
                                break

                if event.type == pygame.MOUSEMOTION:
                    mouse_x, mouse_y = pygame.mouse.get_pos()

                    if not last_hover_rect.collidepoint(mouse_x, mouse_y):
                        last_hover_rect = pygame.Rect(0, 0, 0, 0)

                    for i in self.drops:
                        if i.rect.collidepoint(
                            mouse_x, mouse_y
                        ) and not last_hover_rect.collidepoint(mouse_x, mouse_y):
                            i.mouse_hover()
                            last_hover_rect = i.rect
                            break

                    for i in self.buttons:
                        if i.rect.collidepoint(mouse_x, mouse_y):
                            i.mouse_hover()
                        else:
                            i.mouse_leave()

            SCREEN.blit(BACKGROUND, (0, 0))
            self.buttons.draw(SCREEN)
            self.notifications.update()

            if not self.start_game:
                self.cover.draw()
                self.notifications.draw(SCREEN)
                pygame.display.update()
                continue

            SCREEN.blit(
                PLAYGROUND,
                (PLAYGROUND_OFFSET, PLAYGROUND_OFFSET),
                pygame.Rect(
                    PLAYGROUND_OFFSET,
                    PLAYGROUND_OFFSET,
                    PLAYGROUND_LENGTH,
                    PLAYGROUND_LENGTH,
                ),
            )

            pygame.draw.lines(
                SCREEN,
                (255, 255, 255),
                True,
                [
                    (PLAYGROUND_OFFSET, PLAYGROUND_OFFSET),
                    (PLAYGROUND_OFFSET, PLAYGROUND_LENGTH + PLAYGROUND_OFFSET),
                    (
                        PLAYGROUND_LENGTH + PLAYGROUND_OFFSET,
                        PLAYGROUND_LENGTH + PLAYGROUND_OFFSET,
                    ),
                    (PLAYGROUND_LENGTH + PLAYGROUND_OFFSET, PLAYGROUND_OFFSET),
                ],
                1,
            )

            self.drops.update()
            self.droplets.update()
            self.panel.update(self)

            if len(self.notifications) == 0:
                self.drops.draw(SCREEN)
                self.droplets.draw(SCREEN)
            self.panel.draw(SCREEN)
            self.notifications.draw(SCREEN)

            for drop, droplets in groupcollide(
                self.drops, self.droplets, dokilla=False, dokillb=False
            ).items():
                drop.hit()

                droplets[0].kill()  # many droplets hit same drop, only delete one

                if drop.need_diffuse:
                    self.score += 10
                    self.combo += 1
                    if self.combo in (3, 6, 9, 12, 15, 18, 20):
                        HP_SOUND.play()
                        self.hp += 1
                    elif self.combo > 21:
                        HP_SOUND.play()
                        self.hp += 1
                    Droplet.diffusion(drop.row, drop.col, self.droplets)

            pygame.display.update()

            if len(self.drops) <= 0 and len(self.droplets) == 0:
                self.level += 1
                self.hp += 2
                self.notify(NoticeType.success)
                self._init_grid()

            if len(self.droplets) == 0 and self.hp <= 0:
                self.notify(NoticeType.failed)
                self.start_game = False

        pygame.quit()



from pygame import Font
from pygame.sprite import Sprite
from pygame.color import Color

from ten_drops import PLAYGROUND_OFFSET, PLAYGROUND_LENGTH, FONT_PATH, SCREEN


class Level(Sprite):
    LevelFont = Font(FONT_PATH, size=45)

    def __init__(self, level, *groups):
        super().__init__(*groups)
        self.level = level
        self._update_image()

    def _update_image(self):
        self.image = self.LevelFont.render(f"level {self.level}", True, Color("white"))
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET

    def update(self, game):
        self.level = game.level
        self._update_image()


class Score(Sprite):
    ScoreFont = Font(FONT_PATH, size=40)

    def __init__(self, score, *groups):
        super().__init__(*groups)
        self.score = score
        self._update_image()

    def _update_image(self):
        self.image = self.ScoreFont.render(
            f"score \n{self.score}", True, Color("white")
        )
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET * 4

    def update(self, game):
        self.score = game.score
        self._update_image()


class HP(Sprite):
    HpFont = Font(FONT_PATH, size=40)

    def __init__(self, hp, *groups):
        self.hp = hp
        super().__init__(*groups)

    def _update_image(self):
        self.image = self.HpFont.render(f"drop \n{self.hp}", True, Color("white"))
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET * 7

    def update(self, game):
        self.hp = game.hp
        self._update_image()


class Title(Sprite):
    TitleFont = Font(FONT_PATH, size=120)

    def __init__(self, *groups):
        super().__init__(*groups)
        self.image = self.TitleFont.render("ten drops", True, (51, 255, 255))
        self.rect = self.image.get_rect()
        self.rect.x = (SCREEN.get_width() - self.image.get_width()) / 2
        self.rect.y = SCREEN.get_height() / 10


class About(Sprite):
    AboutFont = Font(FONT_PATH, size=40)

    def __init__(self, *groups):
        super().__init__(*groups)
        self.image = self.AboutFont.render(
            "A ten drops game written in pygame.\n"
            "author: chyok\n"
            "email: chyok@hotmail.com",
            True,
            (32, 32, 32),
        )
        self.rect = self.image.get_rect()
        self.rect.x = (SCREEN.get_width() - self.image.get_width()) / 2
        self.rect.y = SCREEN.get_height() / 3



import pygame

from typing import List, Union
from os.path import join, dirname
from dataclasses import dataclass

from pygame import image, Surface
from pygame.transform import smoothscale, gaussian_blur

__all__ = [
    "SCREEN",
    "GRID_SIZE",
    "BACKGROUND",
    "PLAYGROUND",
    "PLAYGROUND_LENGTH",
    "PLAYGROUND_OFFSET",
    "NOTIFICATION",
    "DROP_IMAGES",
    "DROPLET_IMAGES",
    "FONT_PATH",
    "TEXT_FONT_PATH",
    "BREAK_SOUND",
    "GROW_SOUND",
    "HP_SOUND",
]

pygame.init()

# To solve the error that occurs on some computers without audio output devices
_OPEN_SOUND = True
try:
    pygame.mixer.init()
except Exception as e:  # noqa
    _OPEN_SOUND = False

pygame.display.set_caption("ten drops")

Path = dirname(__file__)
ImageFolderPath = join(Path, "asset", "img")
AudioFolderPath = join(Path, "asset", "audio")

pygame.display.set_icon(pygame.image.load(join(ImageFolderPath, "game.ico")))

cursor = pygame.image.load(join(ImageFolderPath, "cursor.png"))

cursor = pygame.cursors.Cursor((10, 0), cursor)

pygame.mouse.set_cursor(cursor)


@dataclass
class Status:
    action: List[Surface]
    change_action: List[Surface]
    static: Union[Surface, None] = None


class Sound:
    def __init__(self, sound_file: str):
        if _OPEN_SOUND:
            self.sound = pygame.mixer.Sound(join(AudioFolderPath, sound_file))

    def play(self):
        if _OPEN_SOUND:
            self.sound.play()


def get_drop_images() -> List[Status]:
    drop_image_path = join(ImageFolderPath, "drop")
    drop_image_paths = [join(drop_image_path, f"{i}.png") for i in range(134)]

    drop_images = [image.load(i).convert_alpha() for i in drop_image_paths]
    _ = [
        i.fill(pygame.Color("blue"), special_flags=pygame.BLEND_ADD)
        for i in drop_images
    ]

    max_length = max(max(i.get_size()) for i in drop_images[:130])
    radio = PLAYGROUND_LENGTH / GRID_SIZE / max_length

    drop_images = [
        smoothscale(i, (i.get_width() * radio, i.get_height() * radio))
        for i in drop_images
    ]

    return [
        Status(
            action=drop_images[0:20],
            change_action=drop_images[25:31],
            static=drop_images[19],
        ),
        Status(
            action=drop_images[48:66],
            change_action=drop_images[69:76],
            static=drop_images[60],
        ),
        Status(
            action=drop_images[76:90],
            change_action=drop_images[108:115],
            static=drop_images[85],
        ),
        Status(
            action=drop_images[115:130],
            change_action=drop_images[130:134],
            static=drop_images[126],
        ),
    ]


def get_droplet_images() -> List[Status]:
    droplet_image_path = join(ImageFolderPath, "droplet")
    droplet_image_paths = [join(droplet_image_path, f"{i}.png") for i in range(7)]

    droplet_images = [image.load(i).convert_alpha() for i in droplet_image_paths]
    _ = [
        i.fill(pygame.Color("blue"), special_flags=pygame.BLEND_ADD)
        for i in droplet_images
    ]

    max_length = max(max(i.get_size()) for i in droplet_images[:3])

    radio = (
        PLAYGROUND_LENGTH / GRID_SIZE / max_length / 3
    )  # droplet is 3 times smaller than a drop

    droplet_images = [
        smoothscale(i, (i.get_width() * radio, i.get_height() * radio))
        for i in droplet_images
    ]

    return [
        Status(
            action=droplet_images[:3],
            change_action=droplet_images[3:],
            static=droplet_images[1],
        )
    ]


SCREEN = pygame.display.set_mode((800, 600))
GRID_SIZE = 6

FONT_PATH = join(Path, "asset", "font", "kust.ttf")
TEXT_FONT_PATH = join(Path, "asset", "font", "FiraCode-Regular.ttf")

BACKGROUND = smoothscale(
    image.load(join(ImageFolderPath, "background.png")),
    (SCREEN.get_width(), SCREEN.get_height()),
)

NOTIFICATION = smoothscale(
    image.load(join(ImageFolderPath, "notification.png")),
    (SCREEN.get_width() / 1.5, SCREEN.get_height() / 1.5),
)

PLAYGROUND = BACKGROUND.copy()

PLAYGROUND_LENGTH = SCREEN.get_height() * (9 / 10)

PLAYGROUND_OFFSET = (SCREEN.get_height() - PLAYGROUND_LENGTH) / 2

PLAYGROUND = gaussian_blur(PLAYGROUND, 5)

DROP_IMAGES = get_drop_images()

DROPLET_IMAGES = get_droplet_images()

BREAK_SOUND = Sound("break.mp3")
GROW_SOUND = Sound("grow.mp3")
HP_SOUND = Sound("hp.mp3")



from typing import List

from pygame.sprite import Sprite
from pygame.transform import rotate

from ten_drops import DROPLET_IMAGES, BREAK_SOUND
from ten_drops import GRID_SIZE, PLAYGROUND_OFFSET, PLAYGROUND_LENGTH


class Direction:
    Down = (1, 0)
    Up = (-1, 0)
    Left = (0, -1)
    Right = (0, 1)


class Droplet(Sprite):
    def __init__(self, row, col, direction, *groups):
        super().__init__(*groups)
        self.row = row
        self.col = col
        self.direction = direction
        self.speed = 0.13

    def _update_image(self, image):
        if self.direction == Direction.Up:
            image = rotate(image, 90)
        elif self.direction == Direction.Left:
            image = rotate(image, 90 * 2)
        elif self.direction == Direction.Down:
            image = rotate(image, 90 * 3)

        self.image = image

        rect = image.get_rect()

        rect.x = (
            self.col * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.width // 2
            + PLAYGROUND_OFFSET
        )
        rect.y = (
            self.row * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.height // 2
            + PLAYGROUND_OFFSET
        )

        self.rect = rect

    def update(self):
        self.row += self.direction[0] * self.speed
        self.col += self.direction[1] * self.speed
        if any(
            [
                self.row < -0.5,
                self.col < -0.5,
                self.row > GRID_SIZE - 0.5,
                self.col > GRID_SIZE - 0.5,
            ]
        ):
            # The actual animation is offset by half a grid cell
            self.kill()
            return

        self._update_image(DROPLET_IMAGES[0].static)

    @classmethod
    def diffusion(cls, row, col, *group) -> List["Droplet"]:
        BREAK_SOUND.play()
        droplets = []
        for direction in [
            Direction.Down,
            Direction.Up,
            Direction.Left,
            Direction.Right,
        ]:
            droplets.append(Droplet(row, col, direction, *group))
        return droplets



import pygame
from pygame.sprite import Sprite

from ten_drops import (
    DROP_IMAGES,
    GRID_SIZE,
    PLAYGROUND_OFFSET,
    PLAYGROUND_LENGTH,
    GROW_SOUND,
)


class ActionType:
    hover = "hover"
    change = "change"
    static = "static"


class Drop(Sprite):
    def __init__(self, row, col, state=0, *groups):
        super().__init__(*groups)
        self.row = row
        self.col = col
        self.state = state
        self.action_type = ActionType.static
        self.action_count = 0
        self.need_diffuse = False
        self._update_image(DROP_IMAGES[self.state].static)

    def _update_image(self, image):
        self.image = image
        rect = self.image.get_rect()
        rect.x = (
            self.col * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.width // 2
            + PLAYGROUND_OFFSET
        )
        rect.y = (
            self.row * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.height // 2
            + PLAYGROUND_OFFSET
        )
        self.rect = rect

    def click(self):
        if self.action_type == ActionType.change:
            # ignore click when changing
            return

        GROW_SOUND.play()

        if self.state >= len(DROP_IMAGES) - 1:
            self.kill()
            self.need_diffuse = True
            return

        self.state = self.state + 1

        self.action_type = ActionType.change

    def hit(self):
        if self.state > len(DROP_IMAGES) - 1:
            # if many droplets hit same drop, ignore the out of bounds
            return

        elif self.state == len(DROP_IMAGES) - 1:
            self.kill()
            self.need_diffuse = True
            return

        self.state = self.state + 1

        self.action_type = ActionType.change
        return

    def update(self):
        if self.action_type == ActionType.hover:
            all_count = len(DROP_IMAGES[self.state].action)
            if self.action_count < all_count:
                self._update_image(DROP_IMAGES[self.state].action[self.action_count])
                self.action_count += 1
            else:
                self.action_count = 0
                self.action_type = ActionType.static

        elif self.action_type == ActionType.change:
            all_count = len(DROP_IMAGES[self.state - 1].change_action)
            if self.action_count < all_count:
                self._update_image(
                    DROP_IMAGES[self.state - 1].change_action[self.action_count]
                )
                self.action_count += 1
            else:
                self.action_count = 0
                self.action_type = ActionType.hover
        else:
            self._update_image(DROP_IMAGES[self.state].static)

    def mouse_hover(self):
        if self.action_type != ActionType.change:
            self.action_type = ActionType.hover


class DummyDrop(Sprite):
    def __init__(self, row, col, *groups):
        super().__init__(*groups)
        self.row = row
        self.col = col
        self.image = pygame.Surface(
            (
                (PLAYGROUND_LENGTH // GRID_SIZE // 2),
                (PLAYGROUND_LENGTH // GRID_SIZE // 2),
            ),
            pygame.SRCALPHA,
        )
        rect = self.image.get_rect()
        rect.x = (
            self.col * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.width // 2
            + PLAYGROUND_OFFSET
        )
        rect.y = (
            self.row * (PLAYGROUND_LENGTH // GRID_SIZE)
            + (PLAYGROUND_LENGTH // GRID_SIZE // 2)
            - rect.height // 2
            + PLAYGROUND_OFFSET
        )
        self.rect = rect



import pygame

from typing import Dict
from collections import namedtuple

from pygame import Surface, Font
from pygame.color import Color
from pygame.sprite import Sprite

from ten_drops import NOTIFICATION, FONT_PATH, TEXT_FONT_PATH

_NType = namedtuple("_NType", "name font color")


class NoticeType:
    about = "about"
    failed = "failed"
    success = "success"


text_font = Font(TEXT_FONT_PATH, size=16)
text_font.set_bold(True)

TextDict: Dict[str, _NType] = {
    NoticeType.about: _NType("about", text_font, Color("white")),
    NoticeType.failed: _NType("failed", Font(FONT_PATH, size=45), Color("white")),
    NoticeType.success: _NType("failed", Font(FONT_PATH, size=45), Color("white")),
}


class Notice(Sprite):
    def __init__(self, _type, text, *groups):
        super().__init__(*groups)
        self._notification = TextDict[_type]

        self.text_area = Surface(
            (NOTIFICATION.get_width() * 0.8, NOTIFICATION.get_height() * 0.5),
            pygame.SRCALPHA,
        )
        self.image = NOTIFICATION.copy()
        self.rect = self.image.get_rect()
        self.rect.y = -self.rect.height

        text_surface = self._notification.font.render(
            text, True, self._notification.color
        )

        self.text_area.blit(text_surface, (0, 0))
        self.image.blit(
            self.text_area,
            (
                (self.image.get_width() - self.text_area.get_width()) / 1.8,
                NOTIFICATION.get_height() * 0.4,
            ),
        )

    def update(self):
        if self.rect.y < 0:
            self.rect.y += 25



from pygame import Font
from pygame.sprite import Sprite
from pygame.color import Color

from ten_drops import PLAYGROUND_OFFSET, PLAYGROUND_LENGTH, FONT_PATH

NormalFont = Font(FONT_PATH, size=65)
NormalFont.set_italic(True)


class BaseButton(Sprite):
    def __init__(self, *groups):
        super().__init__(*groups)
        self._update_color(Color("white"))

    def mouse_hover(self):
        self._update_color(Color("grey"))

    def mouse_leave(self):
        self._update_color(Color("white"))

    def _update_color(self, color):
        raise NotImplementedError


class StartButton(BaseButton):
    def _update_color(self, color):
        self.image = NormalFont.render("start", True, color)
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET * 13.5


class AboutButton(BaseButton):
    def _update_color(self, color):
        self.image = NormalFont.render("about", True, color)
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET * 15.5


class ExitButton(BaseButton):
    def _update_color(self, color):
        self.image = NormalFont.render("exit", True, color)
        self.rect = self.image.get_rect()
        self.rect.x = PLAYGROUND_LENGTH + PLAYGROUND_OFFSET * 2
        self.rect.y = PLAYGROUND_OFFSET * 17.5
